import CredentialsProvider from "next-auth/providers/credentials";
import { Session, User } from "next-auth";
import { JWT } from "next-auth/jwt";
import type { NextAuthOptions } from "next-auth/index";
import { pages } from "@/config/pages";

export const jwt = async ({ token, user }: { token: JWT; user?: User }) => {
  // first call of jwt function just user object is provided

  if (user?.user?.email) {
    token.user = user
  }

  // on subsequent calls, token is provided and we need to check if it's expired
  // if (token?.exp) {
  //   if (Date.now() / 1000 < token?.exp) return { ...token, ...user };
  // }
  // else if (token?.refreshToken) return token;

  console.log('==jwt==', { token })
  return token;
};

export const session = async ({ session, token }: { session: Session; token: JWT }): Promise<Session> => {
  if (Date.now() / 1000 > token?.exp && token?.exp && Date.now() / 1000 > token?.refreshTokenExpires) {
    return Promise.reject({
      error: new Error("Refresh token has expired. Please log in again to get a new refresh token."),
    });
  }

  session.user = token.user;
  // token.accessTokenExpires = token.exp;

  // session.token = token?.access_token;
  console.log('===session===', session, token)
  return session;
};

export const AuthConfig = {
  providers: [
    CredentialsProvider({
      // The name to display on the sign in form (e.g. 'Sign in with...')
      type: "credentials",
      // The credentials is used to generate a suitable form on the sign in page.
      // You can specify whatever fields you are expecting to be submitted.
      // e.g. domain, username, password, 2FA token, etc.
      // You can pass any HTML attribute to the <input> tag through the object.
      credentials: {},
      async authorize(credentials, req) {
        // You need to provide your own logic here that takes the credentials
        // submitted and returns either a object representing a user or value
        // that is false/null if the credentials are invalid.
        // e.g. return { id: 1, name: 'J Smith', email: 'jsmith@example.com' }
        // You can also use the `req` object to obtain additional parameters
        // (i.e., the request IP address)
        const url = 'http://api.medianet.com.pk' + "/api/v1/auth/login";
        const res = await fetch(url, {
          method: "POST",
          body: JSON.stringify(credentials),
          headers: { "Content-Type": "application/json" },
        });
        const user = await res.json()
        console.log("********************** Testing ********************************");
        console.log(user);


        // If no error and we have user data, return it
        if (user.ok && user || (user.statusCode == 200 || user.statusCode === 201)) {
          return user?.response?.data;
        }

        // Return null if user data could not be retrieved
        // let error = user?.message ?? JSON.stringify(user?.errors) ?? "something went wrong";
        // return { error: JSON.stringify({ error: error, status: user.status }) };
        return null
      },
    }),
  ],
  pages,
  secret: 'querty',
  session: {
    strategy: "jwt",
  },
  callbacks: {
    async signIn({ user, account, profile, email, credentials }) {
      console.log(user.error);
      if (user.error) {
        throw new Error(user.error);
        return false;
      }
      return true;
    },
    async redirect({ url, baseUrl }) {
      // Allows relative callback URLs

      if (url.startsWith("/")) return `${baseUrl}${url}`
      // Allows callback URLs on the same origin
      else if (new URL(url).origin === baseUrl) return url
      return baseUrl
    },
    session,
    jwt,
  },
} satisfies NextAuthOptions;
